#include "stdafx.h"
#include "SpeechEngine.h"
#include "SapiException.h "

using namespace SAPI;


// RXgN^
SpeechEngine::SpeechEngine() :
    m_recoContext( NULL ),
    m_recoGrammar( NULL )
{
}

// fXgN^
SpeechEngine::~SpeechEngine()
{
    if( m_recoContext != NULL )
    {
        // FReLXg
        m_recoContext.Release();
    }

    if( m_recoGrammar != NULL )
    {
        // fBNe[V@
        m_recoGrammar.Release();
    }

    // COM
    ::CoUninitialize();

    ::OutputDebugString( L"SAPIj\r\n" );
}

// 
void SpeechEngine::Init( HWND hWnd, const wchar_t* grammarFile )
{
    HRESULT result;
    CComPtr< ISpRecognizer > recognizer;    // FGW IuWFNg


    // COM
    result = ::CoInitialize( NULL );
    if( FAILED( result ) )
    {
        throw new SapiException( L"COM ̏Ɏs" );
    }

    // FGW IuWFNg𐶐
    result = recognizer.CoCreateInstance( CLSID_SpInprocRecognizer );
    if( FAILED( result ) )
    {
        throw new SapiException( L"FGW IuWFNg̐Ɏs" );
    }

    // FReNXg IuWFNg𐶐
    result = recognizer->CreateRecoContext( &m_recoContext );
    if( FAILED( result ) )
    {
        throw new SapiException( L"FReNXg IuWFNg̐Ɏs" );
    }

    // EBhEY bZ[WFReNXgɓo^
    result = m_recoContext->SetNotifyWindowMessage( hWnd, WM_REC_EVENT, 0, 0 );
    if( FAILED( result ) )
    {
        throw new SapiException( L"EBhEY bZ[W FReNXgւ̓o^Ɏs" );
    }

    // FGWɗvCxg̎ނݒ肷
    const ULONGLONG eventInterest =
        SPFEI( SPEI_RECOGNITION ) |         // 2pXŏIʐ
        SPFEI( SPEI_FALSE_RECOGNITION ) |   // 2pXŏIʐ
        SPFEI( SPEI_SOUND_START ) |         // o (؂osꍇ́A؂oJnʒuo)
        SPFEI( SPEI_SOUND_END );            // ؂oIʒuo

    result = m_recoContext->SetInterest( eventInterest, eventInterest );
    if( FAILED( result ) )
    {
        throw new SapiException( L"FGWɗv Cxg̎ނ̐ݒɎs" );
    }

    // }CN
    InitMicrophone( recognizer );

    // @
    InitGrammar( grammarFile );

    ::OutputDebugString( L"SAPI\r\n" );
}

// }CN
void SpeechEngine::InitMicrophone( CComPtr< ISpRecognizer > &recognizer )
{
    HRESULT result;
    CComPtr< ISpAudio > audio;  // A^C I[fBI Xg[


    // ftHg̃I[fBI IuWFNg𐶐
    result = SpCreateDefaultObjectFromCategoryId( SPCAT_AUDIOIN, &audio );
    if( FAILED( result ) )
    {
        throw new SapiException( L"I[fBI IuWFNg̐Ɏs" );
    }

    // ... ̓Xg[ɉFGWg悤ɐݒ肷
    result = recognizer->SetInput( audio, TRUE );
    if( FAILED( result ) )
    {
        throw new SapiException( L"̓Xg[̐ݒɎs" );
    }
}

// @
void SpeechEngine::InitGrammar( wchar_t const* grammarFile )
{
    HRESULT result;
    const ULONGLONG grammarID = 0;  // @ID


    // XMLɂR}h@𐶐
    result = m_recoContext->CreateGrammar( grammarID, &m_recoGrammar );
    if( FAILED( result ) )
    {
        throw new SapiException( L"R}h@̐Ɏs" );
    }

    if( grammarFile != NULL )
    {
        // t@CfBNe[V@ǂݍ
        result = m_recoGrammar->LoadCmdFromFile( grammarFile, SPLO_STATIC );
        if( FAILED( result ) )
        {
            throw new SapiException( L"t@C̃fBNe[V@̓ǂݍ݂Ɏs" );
        }

        // fBNe[V̕@KANeBuɂ
        result = m_recoGrammar->SetRuleState( NULL, NULL, SPRS_ACTIVE );
        if( FAILED( result ) )
        {
            throw new SapiException( L"fBNe[V̕@KANeBuɂłȂ" );
        }
    }
    else
    {
        // ftHg̃fBNe[V@ǂݍ
        result = m_recoGrammar->LoadDictation( NULL, SPLO_STATIC );
        if( FAILED( result ) )
        {
            throw new SapiException( L"ftHg̃fBNe[V@̓ǂݍ݂Ɏs" );
        }

        // fBNe[ṼgsbNANeBuɂ
        result = m_recoGrammar->SetDictationState( SPRS_ACTIVE );
        if( FAILED( result ) )
        {
            throw new SapiException( L"fBNe[ṼgsbN ANeBuɂłȂ" );
        }
    }
}

// FCxg擾
int SpeechEngine::GetRecognitionEvent()
{
    // L[ɃCxgȂΉȂ
    if( m_spEvent.GetFrom( m_recoContext ) != S_OK )
    {
        return 0;
    }

    // CxgIDԂ
    return m_spEvent.eEventId;
}

// Fʂ擾
BOOL SpeechEngine::GetRecognitionResult( wchar_t* text )
{
    CSpDynamicString dynamicStrig;

    // Fʂ擾
    HRESULT result = m_spEvent.RecoResult()->GetText(
        ( ULONG )SP_GETWHOLEPHRASE,
        ( ULONG )SP_GETWHOLEPHRASE,
        TRUE,
        &dynamicStrig,
        NULL
    );
    if( FAILED( result ) || ( dynamicStrig == NULL ) )
    {
        // 擾ɎsȂ΁A𒆎~
        return FALSE;
    }

    // Fꂽ̌^ϊĎ擾
    wcscpy_s( text, WORD_MAX_SIZE, W2T( dynamicStrig ) );

    return TRUE;
}

// Fꂽ擾
BOOL SpeechEngine::GetRecognizedPronunciation( wchar_t* text )
{
    SPPHRASE* phrase;

    // First-Best̃t[Y擾
    HRESULT result = m_spEvent.RecoResult()->GetPhrase( &phrase );
    if( FAILED( result ) )
    {
        return FALSE;
    }

    // ... ̔擾
    const SPPHONEID* phoneID = phrase->pElements->pszPronunciation;
    if( phoneID == NULL )
    {
        return FALSE;
    }

    // 擾ꂽ̌^ϊ
    wcscpy_s( text, WORD_MAX_SIZE, reinterpret_cast< const wchar_t* >( phoneID ) );

    return TRUE;
}

// Mx擾
BOOL SpeechEngine::GetConfidence( double& value )
{
    SPPHRASE* phrase;

    // First-Best̃t[Y擾
    HRESULT result = m_spEvent.RecoResult()->GetPhrase( &phrase );
    if( FAILED( result ) )
    {
        return FALSE;
    }

    // ... ̐Mx擾
    value = phrase->Rule.SREngineConfidence;

    return TRUE;
}

// Ԃ擾
const wchar_t* SpeechEngine::GetState( int eventID )
{
    switch( eventID )
    {
        case SPEI_SOUND_START:  // Jnꂽ
            return L"F";

        case SPEI_SOUND_END:    // Iꂽ
            return L"I";

        case SPEI_RECOGNITION:  // F
            return L"ҋ@";

        case SPEI_FALSE_RECOGNITION:    // FɎs
            return L"Fs";
    }

    return L"error";
}